@@ -8,7 +8,9 @@ package Protocol::CassandraCQL::ColumnMeta;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
+
+use Carp;
 
 use Protocol::CassandraCQL::Type;
 
@@ -115,6 +117,7 @@ sub column_name
    my $self = shift;
    my ( $idx ) = @_;
 
+   croak "No such column $idx" unless $idx >= 0 and $idx < @{ $self->{columns} };
    my @n = @{ $self->{columns}[$idx] }[0..2];
 
    return @n if wantarray;
@@ -134,6 +137,7 @@ sub column_shortname
    my $self = shift;
    my ( $idx ) = @_;
 
+   croak "No such column $idx" unless $idx >= 0 and $idx < @{ $self->{columns} };
    return $self->{columns}[$idx][3];
 }
 
@@ -149,8 +153,8 @@ sub column_type
    my $self = shift;
    my ( $idx ) = @_;
 
+   croak "No such column $idx" unless $idx >= 0 and $idx < @{ $self->{columns} };
    return $self->{columns}[$idx][4];
-
 }
 
 =head2 $idx = $meta->find_column( $name )
@@ -171,8 +175,11 @@ sub find_column
 
 =head2 @bytes = $meta->encode_data( @data )
 
-Returns a list of encoded bytestrings from the given data according to the type
-of each column.
+Returns a list of encoded bytestrings from the given data according to the
+type of each column. Checks each value is valid; if not throws an exception
+explaining which column failed and why.
+
+An exception is thrown if the wrong number of values is passed.
 
 =cut
 
@@ -181,8 +188,18 @@ sub encode_data
    my $self = shift;
    my @data = @_;
 
+   my $n = @{ $self->{columns} };
+   croak "Too many values" if @data > $n;
+   croak "Not enough values" if @data < $n;
+
+   foreach my $i ( 0 .. $#data ) {
+      my $e = $self->column_type( $i )->validate( $data[$i] ) or next;
+
+      croak "Cannot encode ".$self->column_shortname( $i ).": $e";
+   }
+
    return map { defined $data[$_] ? $self->column_type( $_ )->encode( $data[$_] ) : undef }
-          0 .. $#data;
+          0 .. $n-1;
 }
 
 =head2 @data = $meta->decode_data( @bytes )
