@@ -8,7 +8,7 @@ package Protocol::CassandraCQL::Frame;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Encode qw( encode_utf8 decode_utf8 );
 
@@ -62,6 +62,34 @@ sub new
    bless \$bytes, $class;
 }
 
+=head2 ( $version, $flags, $streamid, $opcode, $frame ) = Protocol::CassandraCQL::Frame->parse( $bytes )
+
+Attempts to parse a complete frame from the given byte string. If it succeeds,
+it returns the header fields and an object containing the frame body. If it
+fails, it returns an empty list.
+
+If successful, it will remove the bytes of the message from the C<$bytes>
+scalar, which must therefore be mutable.
+
+=cut
+
+sub parse
+{
+   my $class = shift;
+   return unless length $_[0] >= 8; # header length
+
+   my $bodylen = unpack( "x4 N", $_[0] );
+   return unless length $_[0] >= 8 + $bodylen;
+
+   # Now committed to extracting a frame
+   my ( $version, $flags, $streamid, $opcode ) = unpack( "C C C C x4", substr $_[0], 0, 8, "" );
+   my $body = substr $_[0], 0, $bodylen, "";
+
+   my $frame = $class->new( $body );
+
+   return ( $version, $flags, $streamid, $opcode, $frame );
+}
+
 =head1 METHODS
 
 =cut
@@ -74,6 +102,22 @@ Returns the byte string currently in the buffer.
 
 sub bytes { ${$_[0]} }
 
+=head2 $bytes = $frame->build( $version, $flags, $streamid, $opcode )
+
+Returns a byte string containing a complete message with the given fields as
+the header and the frame as the body.
+
+=cut
+
+sub build
+{
+   my $self = shift;
+   my ( $version, $flags, $streamid, $opcode ) = @_;
+
+   my $body = $self->bytes;
+   return pack "C C C C N a*", $version, $flags, $streamid, $opcode, length $body, $body;
+}
+
 =head2 $frame->pack_short( $v )
 
 =head2 $v = $frame->unpack_short
